source("{{biopipen_dir}}/utils/misc.R")
library(scCATCH)
library(Seurat)

sobjfile = {{in.sobjfile | r}}
outfile = {{out.outfile | r}}
sccatch_args = {{envs.sccatch_args | r}}

if (is.null(sccatch_args$tissue)) { stop("`envs.sccatch_args.tissue` origin of cells must be defined.") }
if (is.null(sccatch_args$species)) {
    sccatch_args$species = "Human"
}
if (is.null(sccatch_args$marker)) {
    sccatch_args$marker = cellmatch
} else {
    cellmatch = readRDS(sccatch_args$marker)
    sccatch_args$if_use_custom_marker = TRUE
}
if (is.null(sccatch_args$cancer)) {
    sccatch_args$cancer = "Normal"
}
if (is.integer(sccatch_args$use_method)) {
    sccatch_args$use_method = as.character(sccatch_args$use_method)
}

sobj = readRDS(sobjfile)

obj = createscCATCH(data = GetAssayData(sobj), cluster = as.character(Idents(sobj)))
sccatch_args$object = obj

obj = do_call(findmarkergene, sccatch_args)
write.table(
    obj@celltype,
    file = outfile,
    sep = "\t",
    quote = FALSE,
    row.names = FALSE)

celltypes = as.list(obj@celltype$cell_type)
names(celltypes) = obj@celltype$cluster

if (length(celltypes) == 0) {
    warning("No cell types annotated from the database!")
} else {
    celltypes$object = sobj
    sobj = do_call(RenameIdents, celltypes)
    sobj$seurat_clusters = Idents(sobj)

}
saveRDS(sobj, outfile)
